@@ -2,11 +2,21 @@
 Classes of MassGrid and CompositeMap.
 '''
 import pandas as pd
-from mass2chem.search import *
+from scipy import interpolate
+from scipy.ndimage import maximum_filter1d
+import numpy as np
+from mass2chem.search import find_mzdiff_pairs_from_masstracks
 
-from .mass_functions import *
-from .chromatograms import *
-from .peaks import *
+from .mass_functions import (flatten_tuplelist, 
+                             landmark_guided_mapping, 
+                             calculate_selectivity)
+from .chromatograms import (nn_cluster_by_mz_seeds,
+                            rt_lowess_calibration_debug, 
+                            rt_lowess_calibration, 
+                            remap_intensity_track)
+from .peaks import (quick_detect_unique_elution_peak,
+                    batch_deep_detect_elution_peaks,
+                    get_gaussian_peakarea_on_intensity_list)
 from .samples import SimpleSample
 
 
@@ -404,6 +414,12 @@ class CompositeMap:
 
         cal_min_peak_height = self.experiment.parameters['cal_min_peak_height']
         MIN_PEAK_NUM = self.experiment.parameters['peak_number_rt_calibration']
+        NUM_ITERATIONS = self.experiment.parameters['num_lowess_iterations']
+        if self.experiment.parameters['max_retention_shift'] is None:
+            MAX_RETENTION_SHIFT = np.inf
+        else:
+            MAX_RETENTION_SHIFT = self.experiment.parameters['max_retention_shift']
+
         self.good_reference_landmark_peaks = self.set_RT_reference(cal_min_peak_height)
         
         mzDict = dict(self.MassGrid['mz'])
@@ -413,21 +429,29 @@ class CompositeMap:
         for k in mzlist: 
             _comp_dict[k] = basetrack.copy()
 
+        # add to export mz and rtime of good reference landmarks
+        if self.experiment.parameters['debug_rtime_align']:
+            self.export_reference_sample()
+
         for SM in self.experiment.all_samples:
             print("   ", SM.name)
             list_mass_tracks = SM.get_masstracks_and_anchors()
 
-            if self.experiment.parameters['rt_align_on'] and not SM.is_reference:
-                if self.experiment.parameters['debug']:
+            if SM.is_reference:
+                print("\t\tgood_reference_landmark_peaks: ", len(self.good_reference_landmark_peaks))
+            else:
+                if self.experiment.parameters['rt_align_on']:
+                    if self.experiment.parameters['debug_rtime_align']:
+                        cal_func = rt_lowess_calibration_debug
+                    else:
+                        cal_func = rt_lowess_calibration
+
                     self.calibrate_sample_RT(SM, list_mass_tracks, 
-                                        calibration_fuction=rt_lowess_calibration_debug,
+                                        calibration_fuction=cal_func,
                                         cal_min_peak_height=cal_min_peak_height, 
-                                        MIN_PEAK_NUM=MIN_PEAK_NUM)
-                else:
-                    self.calibrate_sample_RT(SM, list_mass_tracks, 
-                                        calibration_fuction=rt_lowess_calibration, 
-                                        cal_min_peak_height=cal_min_peak_height, 
-                                        MIN_PEAK_NUM=MIN_PEAK_NUM)
+                                        MIN_PEAK_NUM=MIN_PEAK_NUM,
+                                        MAX_RETENTION_SHIFT=MAX_RETENTION_SHIFT,
+                                        NUM_ITERATIONS=NUM_ITERATIONS)
 
             for k in mzlist:
                 ref_index = self.MassGrid[SM.name][k]
@@ -461,7 +485,9 @@ class CompositeMap:
                                 list_mass_tracks,
                                 calibration_fuction=rt_lowess_calibration, 
                                 cal_min_peak_height=100000,
-                                MIN_PEAK_NUM=15):
+                                MIN_PEAK_NUM=15,
+                                MAX_RETENTION_SHIFT=np.inf,
+                                NUM_ITERATIONS=3):
         '''
         Calibrate/align retention time per sample.
 
@@ -500,13 +526,14 @@ class CompositeMap:
             When calibration_fuction fails, e.g. inf on lowess_predicted,
             it is assumed that this sample is not amendable to computational alignment,
             and the sample will be attached later without adjusting retention time.
-            To consider to enforce good_landmark_peaks to cover RT range evenly in the future.
+            It will be good to have good_landmark_peaks to cover RT range evenly in the future.
+            Using user-supplied internal standards will be an important option.
         '''
+
         candidate_landmarks = [self.MassGrid[sample.name].values[
                                 p['ref_id_num']] for p in 
                                 self.good_reference_landmark_peaks] # contains NaN
         good_landmark_peaks, selected_reference_landmark_peaks = [], []
-
         for jj in range(len(self.good_reference_landmark_peaks)):
             ii = candidate_landmarks[jj]
             if not pd.isna(ii):
@@ -515,23 +542,25 @@ class CompositeMap:
                 Upeak = quick_detect_unique_elution_peak(this_mass_track['intensity'], 
                             min_peak_height=cal_min_peak_height, 
                             min_fwhm=3, min_prominence_threshold_ratio=0.2)
+                
                 if Upeak:
-                    Upeak.update({'ref_id_num': ii})
-                    good_landmark_peaks.append(Upeak)
-                    selected_reference_landmark_peaks.append(self.good_reference_landmark_peaks[jj])
+                    scan_no_delta = Upeak['apex'] - self.good_reference_landmark_peaks[jj]['apex']
+                    if abs(scan_no_delta) < MAX_RETENTION_SHIFT:
+                        Upeak.update({'ref_id_num': ii})
+                        good_landmark_peaks.append(Upeak)
+                        selected_reference_landmark_peaks.append(self.good_reference_landmark_peaks[jj])
 
         _NN, _CALIBRATED = len(good_landmark_peaks), False
+        print("\t\tgood_landmark_peaks: ", _NN)
+
         sample.rt_landmarks = [p['apex'] for p in good_landmark_peaks]
         # only do RT calibration if MIN_PEAK_NUM is met
         if _NN >  MIN_PEAK_NUM:
-            try:
-                sample.rt_cal_dict, sample.reverse_rt_cal_dict = calibration_fuction( 
-                                            good_landmark_peaks, selected_reference_landmark_peaks, 
-                                            sample.rt_numbers, self.reference_sample.rt_numbers, sample.name,
-                                            self.experiment.parameters['outdir'])
-                _CALIBRATED = True
-            except:         # ValueError:
-                pass
+            sample.rt_cal_dict, sample.reverse_rt_cal_dict = calibration_fuction( 
+                                        good_landmark_peaks, selected_reference_landmark_peaks, 
+                                        sample.rt_numbers, self.reference_sample.rt_numbers, NUM_ITERATIONS, sample.name,
+                                        self.experiment.parameters['outdir'])
+            _CALIBRATED = True
         if not _CALIBRATED:
                 sample.rt_cal_dict, sample.reverse_rt_cal_dict =  {}, {}
                 print("    ~warning~ Faluire in retention time alignment (%d); %s is used without alignment." 
@@ -552,6 +581,12 @@ class CompositeMap:
         Returns
         ------- 
         good_reference_landmark_peaks: [{'ref_id_num': 99, 'apex': 211, 'height': 999999}, ...]
+
+        Note
+        ----
+        Some members in good_reference_landmark_peaks may have the same RT apex.
+        But the redundant numbers should be handled by rt_lowess_calibration, in which .frac is
+        more important for stability.
         '''
         selectivities = calculate_selectivity( self.MassGrid['mz'][self._mz_landmarks_], 
                                                 self.experiment.parameters['mz_tolerance_ppm'])
@@ -570,7 +605,7 @@ class CompositeMap:
                         good_reference_landmark_peaks.append(Upeak)
 
         self.reference_sample.rt_landmarks = [p['apex'] for p in good_reference_landmark_peaks]
-
+        
         return good_reference_landmark_peaks
 
 
@@ -736,3 +771,39 @@ class CompositeMap:
             fList.append( peak_area )
 
         return fList
+    
+    def export_reference_sample(self):
+        """Write mz and retention time of "good" ions to csv in reference sample
+
+        Results
+        -------
+        mz,rtime
+        84.04437446594238,196.3507106869998
+        85.04770363867283,197.100775215
+        90.05493021011353,160.75314731200018
+        100.11204060912132,18.757312656
+        101.11540949344635,19.138889808
+        104.9922667145729,147.4066373920002
+        105.99559181928635,147.7856911519998
+        112.09949165582657,255.0619356640002
+        114.06613251566887,74.11716273600001
+        ......
+
+        The file name would be reference sample name + _mz_rtime_landmarks under export dir
+         
+        """
+        # extendable. could add height and other params
+        mz_landmarks = [self.MassGrid['mz'].values[
+                        p['ref_id_num']] for p in 
+                        self.good_reference_landmark_peaks] 
+        rtime_landmarks = [self.dict_scan_rtime[p['apex']] for p in 
+                        self.good_reference_landmark_peaks]
+        reference_sample_name = self.reference_sample.name
+        import os
+        import csv
+        # example: batch14_MT_20210808_087_mz_rtime_landmarks.csv
+        referece_path = os.path.join(self.experiment.parameters['outdir'], 'export', reference_sample_name + '_mz_rtime_landmarks.csv')
+        with open(referece_path, 'w', newline='') as file:
+            writer = csv.writer(file)
+            writer.writerow(["mz", "rtime"])  #  headers
+            writer.writerows(zip(mz_landmarks, rtime_landmarks)) 
