@@ -3,14 +3,9 @@ import sys
 import json
 import pickle
 
-from mass2chem.search import *
-# jms-metabolite-services
 from jms.dbStructures import knownCompoundDatabase, ExperimentalEcpdDatabase
 
-from .default_parameters import adduct_search_patterns, \
-                                adduct_search_patterns_neg, \
-                                    isotope_search_patterns, \
-                                        extended_adducts
+from .default_parameters import adduct_search_patterns, adduct_search_patterns_neg, isotope_search_patterns, extended_adducts
 
 from .mass_functions import all_mass_paired_mapping
 from .constructors import CompositeMap
@@ -19,7 +14,6 @@ from .json_encoder import NpEncoder
 try:
     import importlib.resources as pkg_resources
 except ImportError:
-    # Shuzhao - what is the purpose of this? I'm assuming this is vestigial and can be removed?
     import importlib_resources as pkg_resources
 
 from . import db
@@ -83,6 +77,9 @@ class ext_Experiment:
             for k,v in self.sample_registry.items():
                 if os.path.basename(self.parameters['reference']) == os.path.basename(v['input_file']):
                     return k
+                elif os.path.basename(self.parameters['reference']) + ".mzML" == os.path.basename(v['input_file']):
+                    return k
+
         elif self.sample_registry:
             L = [(v['number_anchor_mz_pairs'], v['sample_id']) for v in self.sample_registry.values()]
             L.sort(reverse=True)
@@ -356,8 +353,11 @@ class ext_Experiment:
         new_id_start = len(dict_empCpds) + 100000
         for peak in orphans:
             dict_empCpds[new_id_start] = {'interim_id': new_id_start,
-                    'neutral_formula_mass': '', 'neutral_formula': '',
-                    'MS1_pseudo_Spectra': [peak] }
+                    'neutral_formula_mass': '', 
+                    'neutral_formula': '',
+                    'MS1_pseudo_Spectra': [peak],
+                    'ion_relation': None,
+                    'modification': None}
             new_id_start += 1
 
         return dict_empCpds
